#!/bin/sh


version=".v2_1"
suffix=.gz

if [ "$1" = "-F" ]
then
    force=true
    shift
else
    force=false
fi

if [ "$1" = "-c" ]
then
    cdrom=true
    cdpath="$2"
    shift 
    shift
else
    cdrom=false
fi

if [ $# -lt 1 -o ! -f "$1" ]
then
    printf "usage: %s [-F] [-c cdrom-path] configuration_file" $0 >&2
    exit 1
fi

config=$1
export config

HOSTNAME=`hostname`
if $cdrom
then
    BOOTHOST="file://localhost/$cdpath/db"
    BOOTDIR=${TMPDIR:-/var/tmp}/bootups
    UPSBOOTDIR="$cdpath/prd/bootstrap/tarfiles"
else
    BOOTHOST=ftp.fnal.gov
    BOOTDIR=${TMPDIR:-/var/tmp}/bootups
    UPSBOOTDIR=/products/bootstrap/tarfiles
fi

#----------------------------------------------------
# check if we can proceed
#

cannotproceed=false
reason=""

# first check for existing databases

dblist="
    /usr/products/upsdb
    /fnal/ups/db
    /local/ups/db
    /d0usr/products/upsdb
    /d0usr/products/upsdb
    /d0dist/dist/upsdb
    /d0fermi/products/upsdb
    /p/cluster_disk/upsdb
    /products/upsdb
"
for db in  $dblist
do
    if [ -r $db ]
    then
        cannotproceed=true
	base="`echo $db | sed -e 's;\(.*\)/.*;\1;' `"
        reason="$reason
  * ups products database area already exists under $base"
    fi
done

setupslist="
   /usr/local/etc/setups.csh
   /local/ups/etc/setups.csh
   /fnal/ups/etc/setups.csh
"

for setups in  $setupslist
do
    if [ `ls $setups 2>/dev/null | wc -l` -gt 0 ]
    then
        cannotproceed=true
	base="`echo $setups | sed -e 's;\(.*\)/.*;\1;' `"
        reason="$reason
  * ups setups scripts already exist under $base"
    fi
done

# Now we explain...

if $cannotproceed
then
    printf "The ups bootstrap cannot proceed because:%s\n" "$reason" >&2

    if $force
    then
	printf "But since you specified -F, we will proceed anyway...\n"
    else
        printf "These directories must be cleared before we can proceed.\n"
        exit 1
    fi
fi

#----------------------------------------------------
# redirect output. user is now at file descriptor 3

logfile=${TMPDIR:-/var/tmp}/bootups.log
export logfile

exec 3>&1 > $logfile 2>&1

#---------------------------------------------------
# watch the logfile to print a percentage complete
#
(
    if grep 'install.*fullFUE' $config > /dev/null
    then
	expectedlen=1017
    else
	expectedlen=305
    fi
    if grep 'install.*d0cvs' $config > /dev/null
    then
 	expectedlen=`expr $expectedlen + $expectedlen / 7`
    fi

    logfilelen="`ls -s $logfile | sed -e 's/\([0-9][0-9]*\).*/\1/'`"
    while :
    do
       prcnt="`expr $logfilelen \* 100 / $expectedlen 2>/dev/null`" 
       printf "\r%3d%% complete\r" $prcnt >&3
       sleep 2
       logfilelen="`ls -s $logfile | sed -e 's/\([0-9][0-9]*\).*/\1/'`"
    done
) &
percentpid=$!

# ------------------------------
# turn on debugging
#

set -x

export percentpid
trap "kill $percentpid" 0 1 3 15

PATH="/usr/bin:/usr/bsd:$PATH"
export PATH

#-------------------------
# shell function to fetch a file.
#

if $cdrom
then
    fetchfilestdout() {
	(
	    cd $UPSBOOTDIR; 
	    for f in "$@"
	    do
		    if [ -f "$f" ] ; then cat $f; fi
	    done
        )
    }
else
    fetchfilestdout() {
       (
	    printf 'user ftp %s@%s\n' $USER $HOSTNAME
	    printf 'cd %s\n' $UPSBOOTDIR
	    printf 'binary\n'
	    for f in "$@"
	    do
		printf 'get %s -\n' $f
	    done
	    printf 'bye\n'
       ) | ftp -n $BOOTHOST
    }
fi

#------------------------------------
# Mainline
#

unset UPS_EXTRA_DIR

rm -rf ${BOOTDIR}
mkdir -p ${BOOTDIR}
cp $config $BOOTDIR
config="`echo $config | sed -e 's;.*/;;'`"
cd ${BOOTDIR}

if [ ${BOOTDIR} != /var/tmp/bootups ]
then
    ln -s ${BOOTDIR} /var/tmp/bootups
fi

#----------------------------------
# Figure out ups flavor -{1,2.3}
#
flavor1=`uname -s`                            # base flavor of system
relmaj=`uname -r | cut -f 1 -d "."`           # release of system
relmin=`uname -r | cut -f 2 -d "."`           # release of system

# flavor special cases
case "$flavor1" in
IRIX64) 	flavor1="IRIX" ;;
AIX)		tmp=$relmaj; relmaj=$relmin; relmin=$tmp;;
esac

flavor2="${flavor1}+${relmaj}"
flavor3="${flavor1}+${relmaj}.${relmin}"

#-----------------------------------
# get a ups bootstrap tarfile
#

mkdir utils
PATH=${PATH}:${BOOTDIR}/utils

# try for a more specific flavor down to a less specific one
for f in $flavor3 $flavor2 $flavor1
do
    fetchfilestdout $f.gunzip > utils/gunzip
    chmod +x utils/gunzip
    test `cat utils/gunzip | wc -c` -gt 200 || rm utils/gunzip
    fetchfilestdout $f.gtar > utils/tar
    chmod +x utils/tar
    test `cat utils/tar | wc -c` -gt 200 || rm utils/tar
    if [ -x utils/tar ]
    then
        TAR=./utils/tar
    else
        TAR=tar
    fi
    export TAR
    fetchfilestdout $f.tar$suffix |
	    gunzip |
	    $TAR xvf -
    if [ -f $BOOTDIR/etc/setups.sh ]
    then
	break;
    fi
done

#
# get the stage2 script separately -- this should go away later
# once things stabilize
#
fetchfilestdout ../${version}/stage2.sh > stage2.sh

#---------------------------------------------
# run the stage2.sh script, save return status
#
sh -x $BOOTDIR/stage2.sh $config $BOOTDIR
rc=$?

#---------------------------------------------
# finsh up percentage reporting, jump to 100%
#
kill $percentpid
printf "\r100%% complete\n" >&3 
cd /

#--------------------------------------------
# report completion status
#
if [ $rc = 0 ]
then 
    rm -rf $BOOTDIR $logfile &
    printf "Bootstrap succeeded.\n" >&3
else
    printf "Bootstrap failed.  please mail $logfile to ups@fnal.gov\n" >&3
fi
